//------------------------------------------------------------------------------------------------
// OLED_SSD1311 OLED Library
// Copyright (c) 2014, 2015, 2016, 2017, 2018 by Haute Solutions / Ed Van Every (www.haute-solutions.com)
//
// FUNCTION: This Library is intended to control Character OLEDs based on the SSD1311 controller
//
// - Supports I2C Interface Mode Only
// - Autodetects/configures for both Arduino and Mega I2C Pinouts
//
// LICENSE:  This library is free software: you can redistribute it and/or modify
//           it under the terms of the GNU General Public License as published by
//           the Free Software Foundation, either version 3 of the License, or
//           (at your option) any later version.
//
//           This program is distributed in the hope that it will be useful,
//           but WITHOUT ANY WARRANTY; without even the implied warranty of
//           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//           GNU General Public License for more details.
//           <http://www.gnu.org/licenses/>.
//
//------------------------------------------------------------------------------------------------

#define OLED_SSD1311_LIBVER "1.1"

#include <Arduino.h>
#include <Wire.h>

#include "OLED_SSD1311.h"

OLED::OLED(char address) {
	_Address = address;
	_L1_Offset = 0x00;
	_L2_Offset = 0x28;
	
	_Line1[0]='\0';
	_Line2[0]='\0';
}

void OLED::RollDisplay(char *Line2, int Center) {

//  Line2:        "String" = Will display string (truncated/padded/centered as necessary)
//                ""       = Will effectively clear the line
//
//  Center:       0 = No (Left Justify and pad as necessary)
//                1 = Yes (and pad left/right as necessary)
//

	strcpy(_Line1,_Line2);
	DisplayLine(1, _Line1, 0);
	
  if(Line2) DisplayLine(2, Line2, Center);
  else DisplayLine(2, "", 0);
}

void OLED::Display(char *Line1, char *Line2, int Center) {

//  Line1-2:      "String" = Will display string (truncated/padded/centered as necessary)
//                ""       = Will effectively clear the line
//                {NULL}   = Will ignore the line and leave existing info intact
//
//  Center:       0 = No (Left Justify and pad as necessary)
//                1 = Yes (and pad left/right as necessary)
//

  if(Line1) DisplayLine(1, Line1, Center);
  if(Line2) DisplayLine(2, Line2, Center);
}

void OLED::DisplayLine(int Line, char *Text, int Center) {
//  Line:         = Line Number to Display (1-2 as supported)
//
//  Text:         "String" = Will display string (truncated/padded/centered as necessary)
//                ""       = Will effectively clear the line
//                {NULL}   = Will ignore the line and leave existing info intact
//
//  Center:       0 = No (Left Justify and pad as necessary)
//                1 = Yes (and pad left/right as necessary)
int i;
int len;
int char_offset;
char *lp;

  if(Line<1 || Line>OLED_ROWS) return;
  if(Text == NULL) return;
  
  switch (Line) {
  	case 1:
  		lp = _Line1;
  		Command(0x80 + _L1_Offset);
  		break;
  	case 2:
  		lp = _Line2;
  		Command(0x80 + _L2_Offset);
  		break;
  }

  		
  len=strlen(Text);
  if(len == 0) {
    for(i=0; i<OLED_COLS; i++) lp[i]=' ';
  } else if(len > OLED_COLS) {
    strncpy(lp,Text,OLED_COLS);
  } else {
    if(Center) {
      char_offset=(OLED_COLS-len)/2;
      for(i=0; i<char_offset; i++) lp[i]=' ';
      strcpy(lp+char_offset,Text);
      for(i=len+char_offset; i<OLED_COLS; i++) lp[i]=' ';    
    } else {
      strcpy(lp,Text);
      for(i=len; i<OLED_COLS; i++) lp[i]=' ';
    }
  }
  lp[OLED_COLS]='\0';
  
  for(i=0; i<OLED_COLS; i++) Data(lp[i]);
}

void OLED::DisplayChar(int Row, int Col, char C) {
//  Row:         = Row Number to Display (1-2 as supported)
//
//  Col:         = Col Number to Display (1-16 as supported)
//
//  C            = Character to Display
//
char *lp;

  if(Col<1 || Col>OLED_COLS) return;

	switch (Row) {
		case 1:
			lp = _Line1;
  		Command(0x80 + _L1_Offset  + (Col-1));
			break;
		case 2:
			lp = _Line2;
  		Command(0x80 + _L2_Offset  + (Col-1));
			break;
		default:
			return;
	}
	lp[Col-1]=C;
	
  Data(C);
}

void OLED::Clear() {
	_Line1[0]='\0';
	_Line2[0]='\0';
  Command(0x01);
  delay(2);  
}

void OLED::On() {
  Command(0x0C);
  delay(2);  
}

void OLED::Off() {
  Command(0x08);
  delay(2);  
}

void OLED::Init() {
  pinMode(ASCL, OUTPUT);      //set Arduino I2C lines as outputs
  pinMode(ASDA, OUTPUT);      //
  digitalWrite(ASCL, LOW);    //
  digitalWrite(ASDA, LOW);    //

  delay(10);
#ifdef OLED_I2C_ESP32
	Wire.begin(ASDA, ASCL);
#else
	Wire.begin();
#endif  

  delay(100);
  
  Command(0x2A);       // Function Set (RE=1 : Extended Command Set)
  
  	Command(0x71);     // Function Selection A, Internal Vdd Regulator
  	Data(0x00);			   // Disable Internal Vdd Regualtor
  	
  	Command(0x28);     // Function Set (RE=0 : Fundamental Command Set)
  
  Command(0x08);       // Display Off, Cursor Off, Blink Off
  
  Command(0x2A);       // Function Set (RE=1 : Extended Command Set)
  
  	Command(0x79);     // OLED Command Set Enabled (SD=1)
  	
  		Command(0xD5);   // Set Display Clock Divide Ratio/Oscillator Frequency
  		Command(0x70);   // Clock Divide Ratio = 1, Oscillator Frequency = 7
  		
  		Command(0x78);   // OLED Command Set Disabled (SD=0)
  		
  	Command(0x08);     // Extended Function Set (2-lines, 5 Dot Font)
  	Command(0x06);     // COM SEG Direction (COM0->COM31, SEG99->SEG0)

  	Command(0x72);     // Function Selection B, Select Char Gen/ROM
  	Data(0x00);        // CGROM 240, CGRAM 8, ROM A
  
  Command(0x2A);       // Function Set (RE=1 : Extended Command Set)
  
  	Command(0x79);     // OLED Command Set Enabled (SD=1)
  	
  		Command(0xDA);   // Set SEG Pins Hardware Configuration
  		Command(0x10);   // Odd/Even Pin Config, Disable Left/Right Remap

  		Command(0xDC);  // Function Selection C
  		Command(0x00);  // Internal VSL, GPIO pin HiZ Input Disabled (Read as Low)

  		Command(0x81);  // Set Contrast Control
  		Command(0xFF);  // Set To 255 = Max Contrast

  		Command(0xD9);  // Set Phase Length
  		Command(0xF1);  // Phase 2 Period = 15, Phase 1 Period = 1

  		Command(0xDB);  // Set VCOMH Deselect Level
  		Command(0x40);  // 1xVcc

  		Command(0x78);  // OLED Command Set Disabled (SD=0)
  
  	Command(0x28);    // Function Set (RE=0 : Fundamental Command Set)

  Command(0x01);      // Clear Display

  Command(0x80);      // Set DDRAM (Cursor) Address to 0x00

  Command(0x0C);      // Display ON
  
  delay(100);
}

void OLED::Command(unsigned char c) {
  _Packet[0] = 0x00;
  _Packet[1] = c;
  SendPacket(2);
}

void OLED::Data(unsigned char d) {
   _Packet[0] = 0x40;
   _Packet[1] = d;
   SendPacket(2);
}

void OLED::SendPacket(unsigned char x) {
unsigned char i;
  
  Wire.beginTransmission(_Address);
  for(i=0; i<x; i++) Wire.write(_Packet[i]);
  Wire.endTransmission();
}

